# -*- coding: utf-8 -*-
# Migrate fgases_xml
# Run them from within debug mode like so:
#  >>> from Products.Reportek.updates import u20180309_migrate_tr_01A_fs_ex; u20180309_migrate_tr_01A_fs_ex.update(app)

from decimal import Decimal
from Products.Reportek.blob import add_OfsBlobFile
from Products.Reportek.config import DEPLOYMENT_BDR
from Products.Reportek.updates import MigrationBase
import logging
import lxml.etree
import requests
import transaction

logger = logging.getLogger(__name__)
VERSION = 14
APPLIES_TO = [
    DEPLOYMENT_BDR,
]

ALL_XML_LIST = [
    '/fgases/it/9965/envwqkqca/Refrigerant-Italia_F-Gas_Reporting_2016',
    '/fgases/fr/24119/envwqkqkw/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/es/14692/envwqkwva/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/fr/14739/envwqkx4q/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/at/14727/envwqkyfw/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/nl/14733/envwqkypg/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/it/14732/envwqkyyg/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/gr/14707/envwqkzaq/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/ro/14734/envwqkzra/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/lv/14730/envwqkzzg/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/cz/14729/envwqkzhw/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/hu/14787/envwqkzow/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/se/14691/envwqkzwg/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/pt/14809/envwqkz4q/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/de/14786/envwqk0aa/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/se/13207/envwqyyyq/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/it/18189/envwqy33a/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/de/19787/envwqilvg/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/it/13812/envwqi4bw/Fluorinated_gases__F-Gases__reporting__1.xml'
]


def log_msg(msg):
    logger.info(msg)
    print msg


def back_it_up(app, xml):
    if not app.unrestrictedTraverse('/backed_up_1Afs_xml', None):
        app.manage_addFolder('backed_up_1Afs_xml')
    bck_folder = app.unrestrictedTraverse('/backed_up_1Afs_xml')
    xml_filename = xml.split('/')[-1]
    env_path = xml.split(xml_filename)[0]
    env = app.unrestrictedTraverse(env_path)
    folder_name = '_'.join([env.company_id, env_path.split('/')[-2]])
    try:
        bck_folder.manage_addFolder(folder_name)
    except Exception:
        pass
    c_folder = bck_folder.unrestrictedTraverse(folder_name)
    xml_doc = app.unrestrictedTraverse(xml)
    f = getattr(xml_doc, 'data_file')
    fc = f.open()
    c_folder.manage_addFile(xml_filename, file=fc.read())
    fc.close()


def has_backup(app, xml):
    xml_filename = xml.split('/')[-1]
    env_path = xml.split(xml_filename)[0]
    env = app.unrestrictedTraverse(env_path)
    folder_name = '_'.join([env.company_id, env_path.split('/')[-2]])
    bck_xml_file = '/'.join(['/backed_up_1Afs_xml', folder_name, xml_filename])

    if app.unrestrictedTraverse(bck_xml_file, None):
        return True
    return False


def lacks_element(root):
    """Return True if F1_S1_4_ProdImpExp is missing tr_01A_fs and tr_01A_ex"""
    tr_01A = root.xpath("//FGasesReporting//F1_S1_4_ProdImpExp//Gas//tr_01A")
    tr_01A_fs = root.xpath("//FGasesReporting//F1_S1_4_ProdImpExp//Gas//tr_01A_fs")
    if len(tr_01A) != len(tr_01A_fs):
        return True


def add_elements(root):
    missing_01A_fs = """<tr_01A_fs>\n            <Code>01A_fs</Code>\n            <Amount/>\n         </tr_01A_fs>\n         """
    missing_01A_ex = """<tr_01A_ex>\n            <Code>01A_ex</Code>\n            <Amount/>\n         </tr_01A_ex>\n         """
    tr_01A = root.xpath("//FGasesReporting//F1_S1_4_ProdImpExp//Gas//tr_01A")
    for elem in tr_01A:
        elem.addnext(lxml.etree.fromstring(missing_01A_ex))
        elem.addnext(lxml.etree.fromstring(missing_01A_fs))
    return root


def save_xml(old_xml, new_xml):
    new_xml = lxml.etree.tostring(new_xml)
    old_xml.manage_file_upload(file=new_xml,
                               content_type='text/xml',
                               preserve_mtime=True)


def migrate_fgases_xml(app):
    for xml in ALL_XML_LIST:
        xml_filename = xml.split('/')[-1]
        env_path = xml.split(xml_filename)[0]
        try:
            env = app.unrestrictedTraverse(env_path)
            xml_file = app.unrestrictedTraverse(xml)
        except Exception as e:
            log_msg('Unable to get resource: {}'.format(xml))
            continue
        fixed_xml = None
        # back_it_up(app, xml)
        f = getattr(xml_file, 'data_file').open()
        root = lxml.etree.fromstring(f.read())
        f.close()
        if lacks_element(root):
            log_msg("Missing tr_01A_fs and tr_01A_ex for: {}".format(xml))
            if not has_backup(app, xml):
                back_it_up(app, xml)
            fixed_xml = add_elements(root)
            if fixed_xml is not None:
                log_msg("Added missing tr_01A_fs and tr_01A_ex for: {}".format(xml))
                root = fixed_xml
        if fixed_xml is not None:
            save_xml(xml_file, fixed_xml)
        transaction.commit()
    return True


@MigrationBase.checkMigration(__name__)
def update(app, skipMigrationCheck=True):
    if not migrate_fgases_xml(app):
        return

    log_msg('FGases XMLS migration complete')
    return True
