# -*- coding: utf-8 -*-
# Migrate fgases_xml
# Run them from within debug mode like so:
#  >>> from Products.Reportek.updates import u20180322_migrate_ods_gb_gr_country_code;u20180322_migrate_ods_gb_gr_country_code.update(app)

from decimal import Decimal
from Products.Reportek.blob import add_OfsBlobFile
from Products.Reportek.config import DEPLOYMENT_BDR
from Products.Reportek.updates import MigrationBase
from zExceptions import BadRequest
import logging
import lxml.etree
import requests
import transaction

logger = logging.getLogger(__name__)
VERSION = 15
APPLIES_TO = [
    DEPLOYMENT_BDR,
]


ALL_XML_LIST = [
    '/ods/gb/ods30011/envwo6kxa/Ozone_Depleting_Substances_NIL__1.xml',
    '/ods/gb/ods30059/envwo05dq/Ozone_Depleting_Substances_NIL__1.xml',
    '/ods/gb/22888/envwqvw7w/Ozone_Depleting_Substances_NIL__1.xml',
    '/ods/gb/ods23079/envwpf1ew/Ozone_Depleting_Substances__1.xml',
    '/ods/gr/ods20231/envwpfqfg/Ozone_Depleting_Substances__1.xml'
]


def log_msg(msg):
    logger.info(msg)
    print msg


def back_it_up(app, xml):
    if not app.unrestrictedTraverse('/backed_up_xmls_ods_ccode', None):
        app.manage_addFolder('backed_up_xmls_ods_ccode')
    bck_folder = app.unrestrictedTraverse('/backed_up_xmls_ods_ccode')
    xml_filename = xml.split('/')[-1]
    env_path = xml.split(xml_filename)[0]
    env = app.unrestrictedTraverse(env_path)
    folder_name = '_'.join([env.company_id, env_path.split('/')[-2]])
    try:
        bck_folder.manage_addFolder(folder_name)
    except Exception:
        pass
    c_folder = bck_folder.unrestrictedTraverse(folder_name)
    xml_doc = app.unrestrictedTraverse(xml)
    f = getattr(xml_doc, 'data_file')
    fc = f.open()
    try:
        c_folder.manage_addFile(xml_filename, file=fc.read())
    except BadRequest as e:
        log_msg(str(e))
    fc.close()


def save_xml(old_xml, new_xml):
    new_xml = lxml.etree.tostring(new_xml, encoding='UTF-8',
                                  xml_declaration=True)
    old_xml.manage_file_upload(file=new_xml,
                               content_type='text/xml')


def add_feedback(env, fb_id):
    env.manage_addFeedback(id=fb_id,
                     title="Feedback: Country code corrected",
                     feedbacktext="Due to technical IT-related issues in the beginning of March 2018, deliveries for undertakings being situated in Greece and Great Britain were labelled with incorrect country codes. This issue has been handled and the respective reports have been corrected.",
                     automatic=1)


def migrate_ods_gb_gr_cc_xml(app):
    for xml in ALL_XML_LIST:
        xml_filename = xml.split('/')[-1]
        env_path = xml.split(xml_filename)[0]
        try:
            env = app.unrestrictedTraverse(env_path)
        except Exception as e:
            log_msg('Unable to get envelope: {}'.format(env_path))
            continue
        company_id = env.company_id
        try:
            xml_file = app.unrestrictedTraverse(xml)
        except Exception as e:
            log_msg('Unable to get xml file: {}'.format(xml))
            continue
        fixed_xml = None
        back_it_up(app, xml)
        f = getattr(xml_file, 'data_file').open()
        root = lxml.etree.fromstring(f.read())
        f.close()
        c_country = root.xpath("/ODSReporting/GeneralReportData/Company/Country")
        c_country = c_country if c_country else root.xpath("/Reporting/GeneralReportData/Company/Country")
        modified = False
        if c_country:
            c_country = c_country[0]
            c_name = c_country.attrib.get('desc')
            c_code = c_country.text
            if c_name == 'United Kingdom' and c_code != 'GB':
                c_country.text = 'GB'
                modified = True
                log_msg('Changed country code for: {} from: {} to: {}'.format(xml, c_code, 'GB'))
            if c_name == 'Greece' and c_code != 'GR':
                c_country.text = 'GR'
                modified = True
                log_msg('Changed country code for: {} from: {} to: {}'.format(xml, c_code, 'GR'))
        if modified:
            add_feedback(env, 'feedback_correction')
            save_xml(xml_file, root)

        transaction.commit()
    return True


@MigrationBase.checkMigration(__name__)
def update(app, skipMigrationCheck=True):
    if not migrate_ods_gb_gr_cc_xml(app):
        return

    log_msg('ODS GB/GR country code migration complete')
    return True
