# -*- coding: utf-8 -*-
# Migrate fgases_xml
# Run them from within debug mode like so:
#  >>> from Products.Reportek.updates import u20180327_batch_rename;u20180327_batch_rename.update(app)

from decimal import Decimal
from Products.Reportek.blob import add_OfsBlobFile
from Products.Reportek.config import DEPLOYMENT_BDR
from Products.Reportek.updates import MigrationBase
from zExceptions import BadRequest
from DateTime import DateTime
import logging
import lxml.etree
import requests
import transaction

logger = logging.getLogger(__name__)
VERSION = 15
APPLIES_TO = [
    DEPLOYMENT_BDR,
]


def get_documents(app):
    documents = []
    date_range = {
                'query': (
                    DateTime('2018/02/15 00:00:00'), 
                    DateTime('2018/03/27 23:55:59')
                 ),
                'range': 'min:max'}

    for brain in app.Catalog(meta_type='Report Document', reportingdate=date_range):
        d_path = brain.getPath()
        if 'Fluorinated gases (F-Gases) reporting__1.xml' in d_path or 'Ozone Depleting Substances__1.xml' in d_path:
            documents.append(brain.getPath())

    return documents

def log_msg(msg):
    logger.info(msg)
    print msg


def batch_rename(app):
    docs = get_documents(app)
    for d_path in docs:
        doc = app.unrestrictedTraverse(d_path)
        env = doc.getParentNode()
        old_id = doc.id
        new_id = old_id.replace(' ', '_')
        doc.manage_renameObject(old_id, new_id)
        env.reindex_object()
        log_msg('Document: {} renamed from: {} to: {}'.format(d_path, old_id, new_id))

        transaction.commit()
    return True


@MigrationBase.checkMigration(__name__)
def update(app, skipMigrationCheck=True):
    if not batch_rename(app):
        return

    log_msg('ODS GB/GR country code migration complete')
    return True
