# -*- coding: utf-8 -*-
# Migrate fgases_xml
# Run them from within debug mode like so:
#  >>> from Products.Reportek.updates import u20180418_fix_s12_gases; u20180418_fix_s12_gases.update(app)

from decimal import Decimal
from Products.Reportek.blob import add_OfsBlobFile
from Products.Reportek.config import DEPLOYMENT_BDR
from Products.Reportek.updates import MigrationBase
import logging
import lxml.etree
import requests
import transaction

logger = logging.getLogger(__name__)
VERSION = 15
APPLIES_TO = [
    DEPLOYMENT_BDR,
]

# XMLS with 13A problems
XML_LIST_13A = [
    # '/fgases/be/23792/envwqjdya/Fluorinated_gases__F-Gases__reporting__1.xml',
    # '/fgases/bg/23114/envwqfkra/Fluorinated_gases__F-Gases__reporting__1.xml',
    # '/fgases/bg/23114/envwqfioq/Fluorinated_gases__F-Gases__reporting__1.xml',
    # '/fgases/de/26301/envwqqmsq/Fluorinated_gases__F-Gases__reporting__1.xml',
    # '/fgases/be/24706/envwqt7gq/Fluorinated_gases__F-Gases__reporting__1.xml',
    # '/fgases/es/14272/envwpbicg/Fluorinated_gases__F-Gases__reporting__1.xml',
    # '/fgases/es/14272/envwpmbna/Fluorinated_gases__F-Gases__reporting__1.xml',
    # '/fgases/nl/19446/envwo0buw/Fluorinated_gases__F-Gases__reporting__1.xml',
    # '/fgases/de/13233/envwovrtq/Fluorinated_gases__F-Gases__reporting__1.xml',
    # '/fgases/ro/20227/envwp1d2q/Fluorinated_gases__F-Gases__reporting__1.xml',
    # '/fgases/ee/19409/envwqkfqg/Fluorinated_gases__F-Gases__reporting__1.xml',
    # '/fgases/se/14408/envwpw9w/Fluorinated_gases__F-Gases__reporting__1.xml',
    # '/fgases/si/20875/envwoqora/Fluorinated_gases__F-Gases__reporting__1.xml',
    # '/fgases/nl/23222/envwo0dqw/Fluorinated_gases__F-Gases__reporting__1.xml',
    # '/fgases/de/9736/envwqka9q/Fluorinated_gases__F-Gases__reporting__1.xml',
    # '/fgases/pl/22136/envwp5qta/Fluorinated_gases__F-Gases__reporting__1.xml',
    # '/fgases/se/15949/envwpaesg/Fluorinated_gases__F-Gases__reporting__1.xml',
    # '/fgases/nl/22606/envwp1qq/Fluorinated_gases__F-Gases__reporting__1.xml',
    # '/fgases/nl/23371/envwpev9q/Fluorinated_gases__F-Gases__reporting__1.xml',
    # '/fgases/de/26050/envwo6ttg/Fluorinated_gases__F-Gases__reporting__1.xml',
    # '/fgases/cz/17528/envwo0bww/Fluorinated_gases__F-Gases__reporting__1.xml',
    # '/fgases/gb/16207/envwoysuw/Fluorinated_gases__F-Gases__reporting__1.xml',
    # '/fgases/se/14691/envwqkzwg/Fluorinated_gases__F-Gases__reporting__1.xml',
    # '/fgases/pt/20311/envwo6giq/Fluorinated_gases__F-Gases__reporting__1.xml',
    # '/fgases/fr/21471/envwozwww/Fluorinated_gases__F-Gases__reporting__1.xml',
    # '/fgases/de/14445/envwo1uza/Fluorinated_gases__F-Gases__reporting__1.xml',
    # '/fgases/nl/14686/envwp58uq/Fluorinated_gases__F-Gases__reporting__1.xml',
    # '/fgases/de/26050/envwoqtfg/Fluorinated_gases__F-Gases__reporting__1.xml',
    # '/fgases/hu/14564/envwqfx9q/Fluorinated_gases__F-Gases__reporting__1.xml',
    # '/fgases/cy/22687/envwo1jw/Fluorinated_gases__F-Gases__reporting__1.xml',
    # '/fgases/pt/20311/envwoaola/Fluorinated_gases__F-Gases__reporting__1.xml',
    # '/fgases/de/26050/envwnxpdg/Fluorinated_gases__F-Gases__reporting__1.xml',
    # '/fgases/gb/23979/envwpagna/Fluorinated_gases__F-Gases__reporting__1.xml',
]

# XMLS with 12C and 13D problems
XML_LIST_12C = [
    '/fgases/pt/14809/envwqkz4q/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/dk/19111/envwrieiq/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/fr/13768/envwrtldq/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/se/23183/envwnuwza/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/hu/14787/envwqkzow/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/de/14786/envwqk0aa/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/de/23352/envwppo1w/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/se/23465/envwrnfza/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/de/26522/envwo1wcg/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/gb/23748/envwribua/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/gr/24005/envwr0hyg/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/nl/14686/envwp58uq/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/es/24598/envwqka0w/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/ro/22722/envwon7zg/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/cz/14729/envwqkzhw/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/de/22592/envwrtmoq/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/mt/23928/envwrushw/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/be/10086/envwo7pmg/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/fr/27491/envwrjmnq/Fluorinated_gases__F-Gases__reporting__1.xml',
    # Marked as not needed fixing as they are released and rejected
    # '/fgases/fr/27491/envwrjmnq/Fluorinated_gases__F-Gases__reporting__1.xml',
    # '/fgases/de/24844/envwqfxgg/Fluorinated_gases__F-Gases__reporting__1.xml',
    # '/fgases/nl/22930/envwrjndg/Fluorinated_gases__F-Gases__reporting__1.xml',
    # '/fgases/gr/20367/envwrka9w/Fluorinated_gases__F-Gases__reporting__1.xml',
    # '/fgases/pl/18307/envwrpodg/Fluorinated_gases__F-Gases__reporting__1.xml',
    # '/fgases/pl/18307/envwrttmq/Fluorinated_gases__F-Gases__reporting__1.xml',
    # '/fgases/si/19213/envwqen7q/Fluorinated_gases__F-Gases__reporting__1.xml',
    # '/fgases/dk/19111/envwriy_q/Fluorinated_gases__F-Gases__reporting__1.xml',
    # '/fgases/de/24143/envwl1evg/Fluorinated_gases__F-Gases__reporting__2.xml',
    # '/fgases/dk/19111/envwricbq/Fluorinated_gases__F-Gases__reporting__1.xml',
    # '/fgases/dk/19111/envwrisvg/Fluorinated_gases__F-Gases__reporting__1.xml',
    # '/fgases/si/19213/envwqer6w/Fluorinated_gases__F-Gases__reporting__1.xml',
    # '/fgases/de/22592/envwrtura/Fluorinated_gases__F-Gases__reporting__1.xml',
    # '/fgases/fr/27491/envwrsrgq/Fluorinated_gases__F-Gases__reporting__1.xml',
    # '/fgases/de/20299/envwryvkq/Fluorinated_gases__F-Gases__reporting__1.xml',
    # '/fgases/si/19213/envwqeuqa/Fluorinated_gases__F-Gases__reporting__1.xml',
    # '/fgases/de/24844/envwqfljg/Fluorinated_gases__F-Gases__reporting__1.xml',
    # '/fgases/de/20299/envwrzpkg/Fluorinated_gases__F-Gases__reporting__1.xml',
    # '/fgases/de/24143/envwo6noa/Fluorinated_gases__F-Gases__reporting__2.xml',

]

ALL_XML_LIST = list(set(XML_LIST_12C + XML_LIST_13A))


def log_msg(msg):
    logger.info(msg)
    print msg


def back_it_up(app, xml):
    if not app.unrestrictedTraverse('/backed_up_xmls_12c', None):
        app.manage_addFolder('backed_up_xmls_12c')
    bck_folder = app.unrestrictedTraverse('/backed_up_xmls_12c')
    xml_filename = xml.split('/')[-1]
    env_path = xml.split(xml_filename)[0]
    env = app.unrestrictedTraverse(env_path)
    folder_name = '_'.join([env.company_id, env_path.split('/')[-2]])
    try:
        bck_folder.manage_addFolder(folder_name)
    except Exception:
        pass
    c_folder = bck_folder.unrestrictedTraverse(folder_name)
    xml_doc = app.unrestrictedTraverse(xml)
    f = getattr(xml_doc, 'data_file')
    fc = f.open()
    c_folder.manage_addFile(xml_filename, file=fc.read())
    fc.close()


def has_unreported_gases(root):
    """Return True if F8_S12 has a GasCode that is not in ReportedGases"""
    rachp = root.xpath("//Activities//Eq-I-RACHP-HFC//text()")
    if rachp and rachp[0] == 'true':
        gases = root.xpath("//F8_S12//Gas")
        gas_list = root.xpath('//ReportedGases')
        if len(gases) != len(gas_list):
            return True


def remove_unreported_gases(root):
    gases = root.xpath("//F8_S12//Gas")
    reported_gas_ids = root.xpath("//ReportedGases//GasId//text()")
    for gas in gases:
        if gas.find("GasCode").text not in reported_gas_ids:
            if gas.xpath("Totals//tr_12B//text()"):
                log_msg("We have a set value for 12B in an unreported gas")
            gas.getparent().remove(gas)
    return root


def has_gas_clone(root):
    """Return True if F8_S12 has gas clones"""
    gas_ids = root.xpath("//F8_S12//Gas//GasCode//text()")
    if len(gas_ids) != len(set(gas_ids)):
        return True


def has_reportedgas_clone(root):
    """Return True if ReportedGases contains cloned gases"""
    gases = root.xpath("//ReportedGases")
    uniques = []
    for gas in gases:
        gas_code = gas.find("GasId").text
        if gas_code not in uniques:
            uniques.append(gas_code)
        else:
            return True
    return False


def remove_cloned_gases(root):
    gases = root.xpath("//F8_S12//Gas")
    uniques = []
    for gas in gases:
        gas_code = gas.find("GasCode").text
        if gas_code not in uniques:
            uniques.append(gas_code)
        else:
            if gas.xpath("Totals//tr_12B//text()"):
                log_msg("We have a set value for 12B in a cloned gas")
            gas.getparent().remove(gas)

    return root


def calculate_13D(reported_gases, tr_12c, gas_list):
    sum = 0
    for index, gas in enumerate(reported_gases):
        tp = ".//GasId[text()='{}']/../GWP_AR4_AnnexIV//text()".format(gas)
        gwp = gas_list[index].xpath(tp)
        if gwp:
            sum += Decimal(gwp[0]) * Decimal(tr_12c[index])
        else:
            log_msg('Unable to get GWP for GasId: {}'.format(gas))
            return None

    return str(int(round(sum)))


def fix_12C_13D(root):
    tr_12B = root.xpath('//F8_S12//Gas//Totals//tr_12B//text()')
    tr_12C = root.xpath('//F8_S12//Gas//Totals//tr_12C//text()')
    tr_13D = root.xpath('//F9_S12//Totals//tr_13D//Amount//text()')
    gas_list = root.xpath('//ReportedGases')
    if len(tr_12B) != len(tr_12C) or not tr_13D:
        if len(tr_12B) != len(tr_12C):
            for index, tr_12c in enumerate(root.xpath('//F8_S12//Gas//Totals//tr_12C')):
                sum_12a = root.xpath('//F8_S12//Gas//tr_12A//SumOfPartnersAmount//text()')
                if sum_12a:
                    sum_12a = sum_12a[0]
                else:
                    sum_12a = 0
                try:
                    tr_12c_value = Decimal(tr_12B[index]) - Decimal(sum_12a)
                    tr_12c.text = format(tr_12c_value, '.3f')
                except Exception as e:
                    log_msg('Unable to fix 12C')
                    return None
        tr_13D = root.xpath('//F9_S13//Totals//tr_13D//Amount')
        if tr_13D:
            tr_13D = tr_13D[0]
            tr_12C = root.xpath('//F8_S12//Gas//Totals//tr_12C//text()')
            tr_13d_value = calculate_13D(root.xpath('//F8_S12//Gas//GasCode//text()'),
                                         tr_12C,
                                         gas_list)
            if tr_13D.text == '0' or tr_13D.text != tr_13d_value:
                if tr_13d_value:
                    tr_13D.text = tr_13d_value
            else:
                log_msg("13D already has a correct value of: {}".format(tr_13D.text))

            return root
    else:
        log_msg('12C already has values: {}'.format(tr_12C))


def get_fgases_authorisations():
    url = 'https://bdr.eionet.europa.eu/xmlexports/fgases/fgases-authorisations.xml'
    response = requests.get(
        url,
        auth=('bdr_folder_agent', 'WkvRREHj2aou'),
        verify=False)
    return response.content


def get_company_auth_balance(company_id, xml):
    xpath = "//authorisation[@companyId={}]//AuthBalance//text()".format(company_id)
    auth_balance = xml.xpath(xpath)
    if auth_balance:
        return auth_balance[0]


def fix_13A(root, authorisations, company_id):
    s13_auth_balance = root.find('F9_S13//AuthBalance//Amount')
    if not s13_auth_balance.text:
        auth_balance = get_company_auth_balance(company_id, authorisations)
        if auth_balance:
            s13_auth_balance.text = auth_balance
            return root
        else:
            log_msg('Unable to find AuthBalance for: {}'.format(company_id))
    else:
        log_msg('AuthBalance already available for company_id: {} with a value of: {}'.format(company_id, s13_auth_balance.text))


def save_xml(old_xml, new_xml):
    new_xml = lxml.etree.tostring(new_xml)
    old_xml.manage_file_upload(file=new_xml,
                               content_type='text/xml')


def has_12A(root):
    amounts = root.xpath("//F8_S12/Gas/tr_12A/Transaction/Amount/text()")
    transaction_ids = root.xpath("//F8_S12/Gas/tr_12A/Transaction/TransactionID/text()")
    sum = root.xpath("//F8_S12/Gas/tr_12A/SumOfPartnersAmount/text()")
    return sum or amounts or transaction_ids
    # return root.xpath("//F8_S12//Gas//tr_12A//Transaction//Amount")


def add_feedback(env, fb_id):
    env.manage_addFeedback(id=fb_id,
                     title="Feedback: Table 12 and 13 of reporting data on F-gases covering 2017 activities",
                     feedbacktext="<p>Due to technical IT-related issues in the beginning of March 2018, some calculations in Table 12 and 13 that affected the automated preliminary quota compliance assessment of the reported data were possibly incorrect. The issue also related to reports featuring in draft status at the time.</p>"\
"<p>The IT-system was repaired and reporting questionnaires that were already finalised and sent to DG CLIMA/EEA were restored. None of the reported data were touched, only the automated sums of the reported data were corrected.<p>"\
"<p>Data reporters that were affected by the issue, were notified via BDR Helpdesk and encouraged to double check the submission and to verify if the numbers were in accordance with the transactions.</p>",
                     automatic=1)


def migrate_fgases_xml(app):
    # authorisations = lxml.etree.fromstring(get_fgases_authorisations())
    results = []
    for xml in ALL_XML_LIST:
        xml_filename = xml.split('/')[-1]
        env_path = xml.split(xml_filename)[0]
        env = app.unrestrictedTraverse(env_path)
        company_id = env.company_id
        try:
            xml_file = app.unrestrictedTraverse(xml)
        except Exception as e:
            log_msg('Unable to get xml file: {}'.format(xml))
            continue
        fixed_xml = None
        f_13A = False
        f_12C = False
        back_it_up(app, xml)
        f = getattr(xml_file, 'data_file').open()
        root = lxml.etree.fromstring(f.read())
        f.close()
        # if has_12A(root):
        #     print xml

        if has_reportedgas_clone(root):
            log_msg("######Cloned gas in reported gases for: {}".format(xml))
        if has_gas_clone(root):
            log_msg("Cloned gases found for: {}".format(xml))
            fixed_xml = remove_cloned_gases(root)
            if fixed_xml is not None:
                log_msg("Removed cloned gases for F8_S12 in: {}".format(xml))
                root = fixed_xml
        if has_unreported_gases(root):
            fixed_xml = remove_unreported_gases(root)
            if fixed_xml is not None:
                log_msg("Removed unreported gases for F8_S12 in: {}".format(xml))
                root = fixed_xml
        if xml in XML_LIST_12C:
            fixed_xml = fix_12C_13D(root)
            if fixed_xml is not None:
                f_12C = True
                root = fixed_xml
        if xml in XML_LIST_13A:
            fixed_xml = fix_13A(root,
                                authorisations,
                                company_id)
            if fixed_xml is not None:
                f_13A = True
        if fixed_xml is not None:
            save_xml(xml_file, fixed_xml)
            if f_13A or f_12C:
                add_feedback(env, 'feedback_correction')
            if f_13A:
                log_msg('Fixed 13A for: {}'.format(xml))
            if f_12C:
                log_msg('Fixed 12C for: {}'.format(xml))
        else:
            if xml in XML_LIST_12C and not f_12C:
                log_msg('Unable to fix 12C for: {}'.format(xml))
            if xml in XML_LIST_13A and not f_13A:
                log_msg('Unable to fix 13A for: {}'.format(xml))
        transaction.commit()
    return True


@MigrationBase.checkMigration(__name__)
def update(app, skipMigrationCheck=True):
    if not migrate_fgases_xml(app):
        return

    log_msg('FGases XMLS migration complete')
    return True
