# -*- coding: utf-8 -*-
# Migrate fgases_xml
# Run them from within debug mode like so:
#  >>> from Products.Reportek.updates import u20180418_fix_s12_non_hfc; u20180418_fix_s12_non_hfc.update(app)

from decimal import Decimal
from Products.Reportek.blob import add_OfsBlobFile
from Products.Reportek.config import DEPLOYMENT_BDR
from Products.Reportek.updates import MigrationBase
import logging
import lxml.etree
import requests
import transaction

logger = logging.getLogger(__name__)
VERSION = 16
APPLIES_TO = [
    DEPLOYMENT_BDR,
]

DRAFTS = [
    '/fgases/se/14408/envwpw0a/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/be/25643/envwqarwq/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/de/16346/envwp7yvw/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/fr/24157/envwrvgzw/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/fr/24026/envwr3zlw/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/de/19787/envwqigvw/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/ro/20227/envwp1d2q/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/fr/19948/envwrjnfq/Fluorinated_gases__F-Gases__reporting__1.xml'
]

RELEASED = [
    '/fgases/gr/23446/envwrdskw/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/bg/23445/envwryxkq/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/fr/22716/envwrtkta/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/gr/14139/envwrz6qw/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/fr/24082/envwru5tw/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/es/9635/envwros1g/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/fr/21723/envwsnmgg/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/si/23833/envwriaqa/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/be/24230/envwrixyg/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/pl/23870/envwryvha/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/fr/25747/envwr3nlg/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/gr/14430/envwrtm2q/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/fr/24890/envwrualq/LAMOTTEFluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/gr/14650/envwryuea/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/pt/24773/envwqvrga/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/dk/24608/envwsmrxq/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/be/25643/envwqay7w/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/be/26040/envwri83a/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/pt/23796/envwrvtw/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/gb/25965/envww83sg/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/fr/23384/envwryviq/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/it/18209/envwqaamw/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/de/22824/envwrueuw/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/de/18027/envwr_txg/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/se/14691/envwrtk_q/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/es/14692/envwqkwva/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/cy/18085/envwo6fqw/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/fr/23392/envwrtifg/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/gb/20670/envwsyrqq/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/de/23344/envwrznbq/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/fr/24026/envwrz5wg/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/it/18189/envwqy33a/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/gb/14447/envwr0smq/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/nl/14686/envwruaa/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/gb/13670/envwp5xhg/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/de/13878/envwqvr7g/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/hr/21868/envwqpvbw/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/cz/24964/envwrizqw/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/ro/22994/envwrpng/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/it/14235/envwrlj6g/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/be/14949/envwryzia/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/at/26242/envwr078a/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/fr/24133/envwrvnsg/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/fr/24132/envwrvetw/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/pl/22984/envwqpeqw/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/it/14732/envwqkyyg/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/de/14958/envwscfkq/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/pt/18509/envwqwvqg/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/it/14314/envwqeatg/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/gr/23139/envwr4h0q/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/it/13787/envwqudza/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/es/22349/envwrdabw/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/gb/24653/envwsynmg/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/ro/24120/envwro4sa/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/fr/24129/envwrvkxa/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/se/13207/envwqyyyq/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/gr/22887/envwq_g_w/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/hu/14564/envwqfx9q/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/cz/14569/envwrysow/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/es/19682/envwrpfua/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/pt/26135/envwroopg/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/dk/22960/envwruuaq/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/gb/13629/envwsyswq/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/es/23767/envwrpp0q/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/ro/25321/envwr54gw/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/bg/14652/envwry58w/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/at/19769/envwstmfg/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/si/14053/envwrenfq/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/de/14091/envwsii4a/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/be/24075/envwqemng/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/be/25106/envwry39g/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/fr/23577/envwrdavg/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/mt/13915/envwrnodw/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/fr/20309/envwqpiyq/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/gb/20306/envwsufjw/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/it/18109/envwqzesg/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/pl/23097/envwrpjvw/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/ro/21793/envwrterw/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/fr/21372/envwrthhg/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/gb/17506/envwqk1ew/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/cz/14729/envwrocpq/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/at/14727/envwqkyfw/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/it/18112/envwrtua/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/fr/23886/envwrn1wq/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/lv/25003/envwrzpaw/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/gr/22590/envwq_yfw/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/de/16274/envwsdgpw/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/nl/24845/envwqvqwa/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/es/24192/envwri6fg/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/fr/14739/envwqkx4q/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/nl/14733/envwqkypg/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/lv/14730/envwqkzzg/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/ro/14734/envwqkzra/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/bg/23455/envwr1jvq/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/ro/22581/envwr5ugq/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/fr/24119/envwsuoaq/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/fr/24114/envwruxkw/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/gr/14707/envwqkzaq/Fluorinated_gases__F-Gases__reporting__1.xml',
    '/fgases/fr/21432/envwrs8fa/Fluorinated_gases__F-Gases__reporting__1.xml'
]


ALL_XML_LIST = list(set(DRAFTS+RELEASED))


def log_msg(msg):
    logger.info(msg)
    print msg


def back_it_up(app, xml):
    if not app.unrestrictedTraverse('/backed_up_xmls_s1213', None):
        app.manage_addFolder('backed_up_xmls_s1213')
    bck_folder = app.unrestrictedTraverse('/backed_up_xmls_s1213')
    xml_filename = xml.split('/')[-1]
    env_path = xml.split(xml_filename)[0]
    env = app.unrestrictedTraverse(env_path)
    folder_name = '_'.join([env.company_id, env_path.split('/')[-2]])
    try:
        bck_folder.manage_addFolder(folder_name)
    except Exception:
        pass
    c_folder = bck_folder.unrestrictedTraverse(folder_name)
    xml_doc = app.unrestrictedTraverse(xml)
    f = getattr(xml_doc, 'data_file')
    fc = f.open()
    c_folder.manage_addFile(xml_filename, file=fc.read())
    fc.close()


def lacks_element(root):
    """Return True if F1_S1_4_ProdImpExp is missing tr_01A_fs and tr_01A_ex"""
    tr_01A = root.xpath("//FGasesReporting//F1_S1_4_ProdImpExp//Gas//tr_01A")
    tr_01A_fs = root.xpath("//FGasesReporting//F1_S1_4_ProdImpExp//Gas//tr_01A_fs")
    if len(tr_01A) != len(tr_01A_fs):
        return True


def add_elements(root):
    missing_01A_fs = """<tr_01A_fs>\n            <Code>01A_fs</Code>\n            <Amount/>\n         </tr_01A_fs>\n         """
    missing_01A_ex = """<tr_01A_ex>\n            <Code>01A_ex</Code>\n            <Amount/>\n         </tr_01A_ex>\n         """
    tr_01A = root.xpath("//FGasesReporting//F1_S1_4_ProdImpExp//Gas//tr_01A")
    for elem in tr_01A:
        elem.addnext(lxml.etree.fromstring(missing_01A_ex))
        elem.addnext(lxml.etree.fromstring(missing_01A_fs))
    return root


def is_rachp(root):
    rachp = root.xpath("//Activities//Eq-I-RACHP-HFC//text()")
    if rachp and rachp[0] == 'true':
        return True


def is_eq_other(root):
    eq_other = root.xpath("//Activities//Eq-I-other//text()")
    if eq_other and eq_other[0] == 'true':
        return True


def calculate_13B(reported_gases, tr_12b, gas_list):
    sum = 0
    for index, gas in enumerate(reported_gases):
        tp = ".//GasId[text()='{}']/../GWP_AR4_AnnexIV//text()".format(gas)
        gwp = gas_list[index].xpath(tp)
        if gwp:
            sum += Decimal(gwp[0]) * Decimal(tr_12b[index])
        else:
            log_msg('Unable to get GWP for GasId: {}'.format(gas))
            return None

    return str(int(round(sum)))


def calculate_13D(reported_gases, tr_12c, gas_list):
    sum = 0
    for index, gas in enumerate(reported_gases):
        tp = ".//GasId[text()='{}']/../GWP_AR4_AnnexIV//text()".format(gas)
        gwp = gas_list[index].xpath(tp)
        if gwp:
            sum += Decimal(gwp[0]) * Decimal(tr_12c[index])
        else:
            log_msg('Unable to get GWP for GasId: {}'.format(gas))
            return None

    return str(int(round(sum)))


def get_tr11g(s11_gas, gas_id):
    for gas in s11_gas:
        if gas.find('GasCode').text == gas_id:
            return gas.find('tr_11G/Amount').text


def fix_s12(root, xml_file):
    rep_gases = root.xpath('//ReportedGases')
    s12 = root.find('F8_S12')
    s11_gases = root.xpath("//F7_s11EquImportTable/Gas")
    for gas in rep_gases:
        # Create <Gas>
        s12_gas = lxml.etree.SubElement(s12, "Gas")
        # Create <GasCode>
        s12_gascode = lxml.etree.SubElement(s12_gas, "GasCode")
        gas_id = gas.find('GasId').text
        if gas_id:
            s12_gascode.text = gas_id
        # Create tr_12A
        s12_tr_12A = lxml.etree.SubElement(s12_gas, "tr_12A")
        s12_tr_12A_code = lxml.etree.SubElement(s12_tr_12A, "Code")
        s12_tr_12A_sopa = lxml.etree.SubElement(s12_tr_12A, "SumOfPartnersAmount")
        s12_tr_12A_unit = lxml.etree.SubElement(s12_tr_12A, "Unit")
        # Fill tr_12A with a fixed set of predefined values
        s12_tr_12A_code.text = '12A'
        s12_tr_12A_sopa.text = '0.000'
        s12_tr_12A_unit.text = 'metrictonnes'
        # Create Totals
        s12_totals = lxml.etree.SubElement(s12_gas, "Totals")
        # Create tr_12B
        s12_tr_12B = lxml.etree.SubElement(s12_totals, "tr_12B")
        # Create tr_12C
        s12_tr_12C = lxml.etree.SubElement(s12_totals, "tr_12C")

        s12_tr_12B.text = get_tr11g(s11_gases, gas_id)

        tr_12c_value = Decimal(s12_tr_12B.text) - Decimal(s12_tr_12A_sopa.text)
        s12_tr_12C.text = format(tr_12c_value, '.3f')

    if root.find('F8_S12/tr_12A_TradePartners') is None:
        # Create tr_12A_TradePartners
        s12_tr_12A_tp = lxml.etree.SubElement(s12, "tr_12A_TradePartners")

    # Set the value for tr_13B [13B=Sum(12B)]
    tr_13B = root.find('F9_S13/Totals/tr_13B/Amount')
    tr_12B = root.xpath('//F8_S12//Gas//Totals//tr_12B//text()')
    tr_13B.text = calculate_13B(root.xpath('//F8_S12//Gas//GasCode//text()'),
                                tr_12B,
                                rep_gases)

    # Set the value for tr_13C which is sum(12A) so 0 in our cases
    tr_13C = root.find('F9_S13/Totals/tr_13C/Amount')
    tr_13C.text = '0'

    # Set the value for tr_13D [13D=Sum(12C)]
    tr_13D = root.find('F9_S13/Totals/tr_13D/Amount')
    tr_12C = root.xpath('//F8_S12//Gas//Totals//tr_12C//text()')
    tr_13D.text = calculate_13D(root.xpath('//F8_S12//Gas//GasCode//text()'),
                                tr_12C,
                                rep_gases)

    if Decimal(tr_13B.text) != Decimal(tr_13D.text):
        log_msg('We have different values for 13B: {} and 13D: {} for: {}'.format(tr_13B.text, tr_13D.text, xml_file.absolute_url()))

    if Decimal(tr_13B.text) >= 100:
        # Set the Verified
        verified = root.find('F9_S13/Verified')
        verified.text = 'true'
        log_msg('Set verified for section 13 for: {} due to 13B: {} > 100'.format(xml_file.absolute_url(), tr_13B.text))

    return root


def save_xml(old_xml, new_xml):
    new_xml = lxml.etree.tostring(new_xml, encoding='UTF-8',
                                  xml_declaration=True)
    old_xml.manage_file_upload(file=new_xml,
                               content_type='text/xml')


def add_feedback(env, fb_id):
    try:
        env.manage_addFeedback(id=fb_id,
                         title="Feedback: Table 12 and 13 of reporting data on F-gases covering 2017 activities",
                         feedbacktext="<p>Due to technical IT-related issues, some calculations in Table 12 and 13 that affected the automated preliminary quota compliance assessment of the reported data were possibly incorrect. The issue also related to reports featuring in draft status at the time.</p>"\
    "<p>The IT-system was repaired and reporting questionnaires that were already finalised and sent to DG CLIMA/EEA were restored. None of the reported data were touched, only the automated sums of the reported data were corrected. Where applicable (if 13B > 100 t CO2e) the information in section 13 on the need for verification was marked acknowledged.<p>"\
    "<p>Data reporters that were affected by the issue, were notified via BDR Helpdesk and encouraged to double check the submission and to verify if the numbers were in accordance with the transactions.</p>",
                         automatic=1)
    except Exception as e:
        log_msg('We already have a feedback added for: {}'.format(env.absolute_url()))


def migrate_fgases_xml(app):
    issues = []
    for xml in ALL_XML_LIST:
        xml_filename = xml.split('/')[-1]
        env_path = xml.split(xml_filename)[0]
        env = app.unrestrictedTraverse(env_path)
        company_id = env.company_id
        try:
            xml_file = app.unrestrictedTraverse(xml)
        except Exception as e:
            log_msg('Unable to get xml file: {}'.format(xml))
            continue
        fixed_xml = None
        # back_it_up(app, xml)
        f = getattr(xml_file, 'data_file').open()
        root = lxml.etree.fromstring(f.read())
        f.close()
        if root.find('GeneralReportData//TransactionYear').text == '2017':
            if is_rachp(root) and is_eq_other(root):
                issues.append(xml)
            # if lacks_element(root):
            #     fixed_xml = add_elements(root)
            #     if fixed_xml is not None:
            #         log_msg("Added missing tr_01A_fs and tr_01A_ex for: {}".format(xml))
            #         root = fixed_xml
            # if is_rachp(root):
            #     fixed_xml = fix_s12(root, xml_file)
            #     if fixed_xml is not None:
            #         save_xml(xml_file, fixed_xml)
            #         add_feedback(env, 'feedback_correction_s12')
            #         log_msg('Fixed section 12/13 for: {}'.format(xml))
            #     else:
            #         log_msg('Unable to fix 12/13 for: {}'.format(xml))
            # transaction.commit()
    print len(issues)
    print issues
    return True


@MigrationBase.checkMigration(__name__)
def update(app, skipMigrationCheck=True):
    if not migrate_fgases_xml(app):
        return

    log_msg('FGases XMLS s12/s13 migration complete')
    return True
